# Generated by roxygen2: do not edit by hand

export(add_index_variables)
export(build_jags_inputs)
export(build_spline_basis)
export(clean_fp_source_data)
export(compute_T_star)
export(extract_mcmsector_params)
export(fix_region_names)
export(replace_small_or_missing_se)
export(run_preprocessing_pipeline)
export(simulate_posterior_proportions)
export(standardize_country_names)
export(summarise_posterior_proportions)
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,lag)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(haven,as_factor)
importFrom(haven,read_dta)
importFrom(labelled,val_labels)
importFrom(magrittr,"%>%")
importFrom(openxlsx,write.xlsx)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,vcov)
importFrom(survey,svyby)
importFrom(survey,svyciprop)
importFrom(survey,svydesign)
importFrom(survey,svymean)
importFrom(survey,svytable)
importFrom(tibble,as_tibble)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,separate)
importFrom(tidyr,spread)
